import pyslim
import tszip
import tskit
import numpy as np

input_dir = '/Users/shyamie/Desktop/Archive'
tree1 = pyslim.update(tszip.decompress(input_dir + '/tree1.trees.tsz'))
tree2 = pyslim.update(tszip.decompress(input_dir + '/tree2.trees.tsz'))
div_time = 187

# MERGE TREES
# create lists of slim_ids
tree1_slim_ids = np.array([i.metadata['slim_id'] for i in tree1.nodes()])
tree2_slim_ids = np.array([i.metadata['slim_id'] for i in tree2.nodes()])
# create node map
node_map = np.full(tree1.num_nodes, tskit.NULL)
# identify nodes in other_tree that existed before the trees split
is_alive_before_div = (tree1.nodes_time > div_time) # ** should this be > or >= ??
# identify nodes with common slim_ids
is_common = np.isin(tree1_slim_ids, tree2_slim_ids)
# identify nodes that fulfil both requirements
is_both = np.logical_and(is_alive_before_div, is_common)
# match nodes
tree2_slim_ids_sorted = np.argsort(tree2_slim_ids)
matches = np.searchsorted(tree2_slim_ids, tree1_slim_ids[is_both], side='left', sorter=tree2_slim_ids_sorted)
node_map[is_both] = tree2_slim_ids_sorted[matches]
# do not match nodes where the ages of tree 1 and tree 2 do not match
age_mismatches = np.array([i for i in range(tree1.num_nodes) if node_map[i] != -1 and tree1.nodes_time[i] != tree2.nodes_time[node_map[i]]])
if len(age_mismatches) > 0:
	node_map[age_mismatches] = tskit.NULL
# merge and simplify
merged_tree = tree2.union(tree1, node_map, check_shared_equality=False).simplify()

# Recapitate merged tree
pyslim.recapitate(merged_tree, recombination_rate = 1e-8, ancestral_Ne = 3e4)